/*-------------------------------------------------------------------------*/
#define MAXCELLS 128      /* max. number of reaction cells */
#define MAXELT 24         /* maximum number of BEM elements */
#define MAXCPL 256        /* maximum number of BEM complexes */
#define MAXSLT 12         /* maximum number of BEM salts */
#define MAXIEX 1          /* maximum number of BEM ion exchangers */
#define MAXGAS 1          /* maximum number of Gases (CO2 ...) */
#define MAXIDNAM 8        /* max. length of cell name */
#define MAXELTNAM 16      /* maximum length of element name */
#define MAXCPLNAM 48      /* maximum length of complex name */
#define MAXSLTNAM 48      /* maximum length of salt name */
#define MAXGASNAM 16      /* maximum length of gas name */
#define MAXPARTS 12       /* maximum number of elements in comlex or salt */
#define MAXCHRG 12        /* max. ionic charge */
#define MAXSPFPAR 6   /* max. number of parameters for solubil.prod.funct.*/
#define MAXPUTVAR 64  /* max. number of put - variables */

#define MAXDIM (MAXELT+MAXSLT+MAXIEX) /* max. dimension of object function */

#define MAXINREC 256  /* maximum length of input records (model file etc.) */ 
#define MAXWORDS 128  /* max. number of words in input line */
#define MAXFILNAM 64  /* max. length of filenames */
#define MAXINTSTR 12  /* max. length of integer field in input */
#define MAXFPERS  10  /* max. restarts with random disturbation after FPE */

typedef struct{          /* BEM element */
  char name[MAXELTNAM];
  double ctot,ctoti,csng,cttl,pact;
  double ctl0,estsng;
  int chrg;
  short num;
} ELT;

typedef struct{                    /* BEM gas (CO2)                   */
  char name[MAXGASNAM];
  double khen,pfix;
  int cpnum,elnum;
} GAS;

typedef struct{                    /* BEM complex */
  char name[MAXCPLNAM];
  ELT *el[MAXPARTS];
  int stcf[MAXPARTS],nelt,chrg;
  double ccpl,lnkds;
  GAS *gas;
} CPL;

typedef struct{                    /* BEM salt    */
  char name[MAXSLTNAM],state;
  ELT *el[MAXPARTS];
  int stcf[MAXPARTS],nelt,idspf;
  double cslt,spfpar[MAXSPFPAR],cslt0,estslt,coff,wpt,wptfix,csltt ;
} SLT;

typedef struct{                    /* BEM cation exchange complex    */
  char spec;
  double lnksel[MAXELT],xs[MAXELT],estxsh,xcap;
  double inparl[MAXELT],inpars[MAXELT];
} CEC;


typedef struct{                      /* numerical set */
double xarg[MAXDIM],scl[MAXDIM];
} NUS;


typedef struct{                     /* BEM reaction cell */
  ELT *el[MAXELT];
  CPL *cp[MAXCPL];
  SLT *sl[MAXSLT];
  GAS *gs[MAXGAS];
  CEC *xc;
  char id[MAXIDNAM];
  int step,ncpl,nelt,nslt,niex,ngas;
  int it1cnt,it1max,it2cnt,it2max,itrace;
  double theta0,theta,rho,sigma,sigma0,ionstr;
  double estph,eststr,relerr,lsierr,alf,dalf;
  double lnkw;
  double temp; /* temperature in K */
  void *putpnt[MAXPUTVAR];
  NUS nus0,nus1,nus2;
} CLL;

typedef struct{
  FILE  *fil;   /* pointer to file */
  char  *nam;   /* name of file */
} FIL;

typedef struct{               /* indicators of command line options */
  int s,i,q,Q;
} OPT;

#define RGAS 0.081004 /* (bar*L)/(K*mol) */

#define CALL(r,x) r=x;if(r<-31)return r
#define STR(t) #t
#define QUOTE(t) STR(t)

#include "init.h"
#include "util.h"
#include "chem.h"
#include "findeq.h"
#include "sltmng.h"
#include "cttmng.h"
#include "output.h"
#include "lspl.h"

/* ........................................ global variables ...............*/
OPT Op;      /* the command line option indicators */
CLL *Dir[MAXCELLS]; /* the directory of reaction cells  */
int Ncells;     /* number of reaction cells */ 
CLL *Cl;   /* a pointer to the reaction cell, which is worked on at present */
FIL Tat;        /* the .t - file */
FIL Mod;        /* a .m - file */
FIL Lgf;        /* the log (iteration progress report) file */
FIL Ifil;        /* results of numerical procedure for quick replay (write) */
FIL Qfil;        /* results of numerical procedure for quick replay (read) */
FIL Sfil;        /* Output in "show" tabular format */
FIL Res;        /* output as defined by put statements */
FILE *Flowfil;   /* temporary file for experiment-specifications */
FILE *Putsfil;   /* temporary file for put statements            */
int Fpecnt;     /* counter of SIGFPEs */
char *Actfil;   /* the actual file studied (for error messages) */
double Deltat;  /* time per step  */
int Puts[MAXPUTVAR]; /* key numbers for put variables */
int Nputs;      /* Number of put statements */
int Qind;       /* indicates that quick replay is tried */
