/*   BEM = Batch Equilibrium Model (calculates chemical equilibira)
     Copyright (C) 1996  J. Prenzel (jprenze@gwdg.de)
     
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
     as published by the Free Software Foundation; either version 2
     of the License, or (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         */

#include <stdio.h>
#include <math.h>
#include <signal.h>
#include <errno.h>
#include <stdlib.h>

#include "bem.h"

#define TTI(x) printf(">>>>>>>>>>>>>>> %d\n",x);
#define TTX(x) printf("............... %g\n",x);

static int sclset(double scl[]);
static int sclrnd(double scl[]);
static int dfmfpc(int *n, double *x, double *f, 
	double *g, double *est, double *eps, int *limit, 
	int *ier, double *h);
static int sumsq(int *, double *, double *, double *, int *);
static void fpehdl(int code);


/*............................................... findeq................*/
int findeq(){
  double ssq,dssq[MAXDIM],wrk[MAXDIM*(MAXDIM+7)/2];
  double est=0., eps=1.e-13;
  int limit=100,ier; 
  int nx,rc,nfpe,kmax;

  nx=Cl->nelt + Cl->niex + Cl->nslt;
  Cl->it1cnt=0;
  nfpe=0;

  if(Cl->step>0){
    kmax=Cl->it1max;
  }
  else{
    kmax=Cl->it1max*10;
  }

  Cl->nus2 = Cl->nus0;

  while(1){
    signal(SIGFPE,fpehdl);
    Fpecnt=0;
    errno=0;
    Cl->nus1=Cl->nus2;

    sumsq(&nx,Cl->nus2.xarg,&ssq,dssq,&ier);
    rc=ctrleq();
    if(rc == 0){
      if(Cl->itrace>2)itrep(3,ier,rc,0,ssq,0.,0.);
      Cl->nus0=Cl->nus2;
      return 0;
    }

    dfmfpc(&nx,Cl->nus2.xarg,&ssq,dssq,&est,&eps,&limit,&ier,wrk);
    signal(SIGFPE,SIG_DFL);
    if(ier == 3){
      if(Cl->itrace>2)itrep(2,ier,0,0,ssq,0.,0.);
      if(Cl->it1cnt > kmax) return 3;
      nfpe++;
      if(nfpe>MAXFPERS) return 3;
      Cl->nus2=Cl->nus1;
      sclrnd(Cl->nus2.scl);
      continue;
    }
    rc=ctrleq();
    if(Cl->itrace>2)itrep(3,ier,rc,0,ssq,0.,0.);
    if(rc == 0){
      Cl->nus0=Cl->nus2;
      return 0;
    }
    if(Cl->it1cnt > Cl->it1max) return rc;
    sclset(Cl->nus2.scl);
    sclrnd(Cl->nus2.scl);
   }
  return 0;
}

/*.................................................. setx0 ...........*/
int setx0(){

  int i,k,m,nx,ieri,tracold;
  double a,c,d,s;
  double lngam[MAXCHRG+1],dsdx[MAXDIM];

  nx=Cl->nelt + Cl->niex + Cl->nslt;
  signal(SIGFPE,fpehdl);
  Fpecnt=0;
  errno=0;
 
  d=Cl->eststr;
  c=sqrt(d);
  a=-0.5*(c/(1.0+c) - 0.3*d)*M_LN10; /* Davies' equation used here */
  for(i=0;i<=MAXCHRG;i++){
    lngam[i]=a*i*i;
  }
  Cl->nus0.xarg[0]=Cl->estph*(-M_LN10);
  Cl->nus0.xarg[1]=log(Cl->eststr);
  for(i=2;i < Cl->nelt;i++){
    m=abs(Cl->el[i]->chrg);
    Cl->nus0.xarg[i]=log(Cl->el[i]->estsng) + lngam[m];
  }

  if(Cl->niex>0){
    if(Cl->xc->xcap > 0.){
      k=Cl->nelt;
      Cl->nus0.xarg[k]=log(Cl->xc->estxsh);
    }
  }

  for(i=0;i<Cl->nslt;i++){
    k=Cl->nelt+Cl->niex+i;
    if(Cl->sl[i]->state == '+'){
      Cl->nus0.xarg[k]=log(Cl->sl[i]->estslt);
    }
    else{
      Cl->nus0.xarg[k]=0.;
    }
  }

  Cl->theta=Cl->theta0;
  Cl->sigma=Cl->sigma0;

  tracold=Cl->itrace;
  Cl->itrace=0;
  sumsq(&nx, Cl->nus0.xarg, &s, dsdx, &ieri);
  signal(SIGFPE,SIG_DFL);
  if(ieri != 0){
    bemerr(30,"","","",ieri);
    return -32;
  }
  sclset(Cl->nus0.scl);
  Cl->itrace=tracold;;

  return 0;
}

/*.............................................. sclset ................*/
int sclset(double scl[]){
  double sani,old[MAXDIM]  ;
  int i,z,k;

  for(i=0;i<MAXDIM;i++){
    old[i]=scl[i];
  }
  signal(SIGFPE,fpehdl);
  Fpecnt=0;
  errno=0;

  sani=0;
  for(i=0;i<Cl->nelt;i++){
    z=Cl->el[i]->chrg;
    if(z<0) sani -= z*Cl->el[i]->csng;
  }
  for(i=0;i<Cl->ncpl;i++){
    z=Cl->cp[i]->chrg;
    if(z<0) sani -= z*Cl->cp[i]->ccpl;
  }
  scl[0]=1./sani;
  scl[1]=1./Cl->ionstr;
  for(i=2;i<Cl->nelt;i++){
    scl[i]=1./Cl->el[i]->ctoti;
  }
  k=Cl->nelt;
  for(i=0;i<Cl->niex;i++){
    scl[k]=1.;
    k++;
  }
  for(i=0;i<Cl->nslt;i++){
    scl[k]=1./fabs(Cl->sl[i]->spfpar[0]);
    k++;
  }

  for(i=0;i<Cl->ngas;i++){
    if(Cl->gs[i]->pfix >0){    
      scl[Cl->gs[i]->elnum] = 1./(Cl->gs[i]->pfix * Cl->gs[i]->khen);
    }
  }

  if(Cl->step > 0){ /* Bedingung neu 16.12.95 (zufllige Erfahrungssache) */  
    for(i=0;i < Cl->nelt + Cl->niex + Cl->nslt;i++){
      scl[i] *= scl[i];
    } 
  }

  if(Fpecnt > 0 || errno != 0){
    for(i=0;i<MAXDIM;i++){
      scl[i]=old[i];
    }
  }
  signal(SIGFPE,SIG_DFL);

  return 0;
}

/*.............................................. sclrnd ................*/
int sclrnd(double scl[]){
#define RND (1.1 - 0.2 *((double) rand() / (double) RAND_MAX))
  int i,k;

  for(i=0;i<Cl->nelt;i++){
    scl[i] *= RND;
  }
  k=Cl->nelt;
  for(i=0;i<Cl->niex;i++){
    scl[k] *= RND;
    k++;
  }
  for(i=0;i<Cl->nslt;i++){
    scl[k] *= RND;
    k++;
  }
return 0;
}



/*........................................... sumsq ...................
  makes sum of squares from chemical equations (fxdfdx), is object of
  optimizing routine (dfmfpc)
*/
int sumsq(int *nx, double x[], double *s, double dsdx[], int *ieri){


  double f[MAXDIM], df[MAXDIM][MAXDIM];
  int i,ii;
  Cl->it1cnt++;
  *ieri=0;
  *s=0.;
  fxdfdx(x,*nx,f,df);
  if(Fpecnt > 0 || errno != 0){ /* react to SIGFPEs */
    *ieri=3;
    return 0;
  }


  for(i=0; i < *nx; i++){
    *s += f[i]*f[i]*Cl->nus2.scl[i];
    dsdx[i]=0.;   
    for(ii=0; ii < *nx; ii++){
      dsdx[i] += 2*f[ii]*df[ii][i]*Cl->nus2.scl[ii];
    }
  }

  if(Cl->itrace >= 4){
    if((Cl->it1cnt % 100 == 0) || Cl->it1cnt < 4){
      itrep(1,0,0,0,*s,0.,0.);
      /*    xlist1(tst,6);
	    xlist1(x,*nx);
	    xlist1(f,*nx);
	    xlist1(dsdx,*nx); */
    }
  }

  if(Fpecnt > 0 || errno != 0){ /* react to SIGFPEs */
    *ieri=3;
   }

  return 0;
} 

/* .............................................. fpehdl .............
   SIGFPE - handler, just counts SIGFPEs */
void fpehdl(int code){
  signal(SIGFPE,fpehdl);
  Fpecnt++;
  return;
}


/* dfmfpc.f -- translated by f2c (version 19940714).
*/
#include "f2c.h"

/* changed version of DFMFP */
/* changes by J. Prenzel marked by "!JP" */
/* prepared for C-bem (f2c) oct 1995 */
/*     .................................................................. */

/*        SUBROUTINE DFMFP */

/*        PURPOSE */
/*           TO FIND A LOCAL MINIMUM OF A FUNCTION OF SEVERAL VARIABLES */
/*           BY THE METHOD OF FLETCHER AND POWELL */

/*        USAGE */
/*           CALL DFMFP(FUNCT,N,X,F,G,EST,EPS,LIMIT,IER,H) */

/*        DESCRIPTION OF PARAMETERS */
/*           FUNCT  - USER-WRITTEN SUBROUTINE CONCERNING THE FUNCTION TO */
/*                    BE MINIMIZED. IT MUST BE OF THE FORM */
/*                    SUBROUTINE FUNCT(N,ARG,VAL,GRAD,*) */
/*                    AND MUST SERVE THE FOLLOWING PURPOSE */
/*                    FOR EACH N-DIMENSIONAL ARGUMENT VECTOR  ARG, */
/*                    FUNCTION VALUE AND GRADIENT VECTOR MUST BE COMPUTED */
/*                    AND, ON RETURN, STORED IN VAL AND GRAD RESPECTIVELY */
/*                    ARG,VAL AND GRAD MUST BE OF DOUBLE PRECISION. */
/*           N      - NUMBER OF VARIABLES */
/*           X      - VECTOR OF DIMENSION N CONTAINING THE INITIAL */
/*                    ARGUMENT WHERE THE ITERATION STARTS. ON RETURN, */
/*                    X HOLDS THE ARGUMENT CORRESPONDING TO THE */
/*                    COMPUTED MINIMUM FUNCTION VALUE */
/*                    DOUBLE PRECISION VECTOR. */
/*           F      - SINGLE VARIABLE CONTAINING THE MINIMUM FUNCTION */
/*                    VALUE ON RETURN, I.E. F=F(X). */
/*                    DOUBLE PRECISION VARIABLE. */
/*           G      - VECTOR OF DIMENSION N CONTAINING THE GRADIENT */
/*                    VECTOR CORRESPONDING TO THE MINIMUM ON RETURN, */
/*                    I.E. G=G(X). */
/*                    DOUBLE PRECISION VECTOR. */
/*           EST    - IS AN ESTIMATE OF THE MINIMUM FUNCTION VALUE. */
/*                    SINGLE PRECISION VARIABLE. */
/*           EPS    - TESTVALUE REPRESENTING THE EXPECTED ABSOLUTE ERROR. */
/*                    A REASONABLE CHOICE IS 10**(-16), I.E. */
/*                    SOMEWHAT GREATER THAN 10**(-D), WHERE D IS THE */
/*                    NUMBER OF SIGNIFICANT DIGITS IN FLOATING POINT */
/*                    REPRESENTATION. */
/*                    SINGLE PRECISION VARIABLE. */
/*           LIMIT  - MAXIMUM NUMBER OF ITERATIONS. */
/*           IER    - ERROR PARAMETER */
/*                    IER = 0 MEANS CONVERGENCE WAS OBTAINED */
/*                    IER = 1 MEANS NO CONVERGENCE IN LIMIT ITERATIONS */
/*                    IER =-1 MEANS ERRORS IN GRADIENT CALCULATION */
/*                    IER = 2 MEANS LINEAR SEARCH TECHNIQUE INDICATES */
/*                            IT IS LIKELY THAT THERE EXISTS NO MINIMUM. */
/*                   ier = 3 means invalid results, floating point      !JP*/
/*                           exceptions (overflow, underflow) did occur !JP*/
/*           H      - WORKING STORAGE OF DIMENSION N*(N+7)/2. */
/*                    DOUBLE PRECISION ARRAY. */

/*        REMARKS */
/*            I) THE SUBROUTINE NAME REPLACING THE DUMMY ARGUMENT  FUNCT */
/*               MUST BE DECLARED AS EXTERNAL IN THE CALLING PROGRAM. */
/*           II) IER IS SET TO 2 IF , STEPPING IN ONE OF THE COMPUTED */
/*               DIRECTIONS, THE FUNCTION WILL NEVER INCREASE WITHIN */
/*               A TOLERABLE RANGE OF ARGUMENT. */
/*               IER = 2 MAY OCCUR ALSO IF THE INTERVAL WHERE F */
/*               INCREASES IS SMALL AND THE INITIAL ARGUMENT WAS */
/*               RELATIVELY FAR AWAY FROM THE MINIMUM SUCH THAT THE */
/*               MINIMUM WAS OVERLEAPED. THIS IS DUE TO THE SEARCH */
/*               TECHNIQUE WHICH DOUBLES THE STEPSIZE UNTIL A POINT */
/*               IS FOUND WHERE THE FUNCTION INCREASES. */

/*        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED */
/*           FUNCT */

/*        METHOD */
/*           THE METHOD IS DESCRIBED IN THE FOLLOWING ARTICLE */
/*           R. FLETCHER AND M.J.D. POWELL, A RAPID DESCENT METHOD FOR */
/*           MINIMIZATION, */
/*           COMPUTER JOURNAL VOL.6, ISS. 2, 1963, PP.163-168. */

/*     .................................................................. */

int dfmfpc(int *n, doublereal *x, doublereal *f, 
	doublereal *g, doublereal *est, doublereal *eps, int *limit, 
	int *ier, doublereal *h)
{
    /* System generated locals */
    integer i__1, i__2;
    doublereal d__1, d__2;

    /* Builtin functions */
    double sqrt(doublereal);

    /* Local variables */
    static doublereal alfa, oldf;
    static int ieri;
    static doublereal gnrm, hnrm, ambda;
    static int i, j, k, l;
    static doublereal dalfa, t, w, z;
    static int kount, n2, n3;
    static int n31, kl, nj;
    static doublereal dx, dy, fx, fy;


/*        DIMENSIONED DUMMY VARIABLES */
/*cc      DOUBLE PRECISION X,F,FX,FY,OLDF,HNRM,GNRM,H,G,DX,DY,ALFA,DALFA, 
!JP*/
/*cc     1AMBDA,T,Z,W                                                     
!JP*/

/*        COMPUTE FUNCTION VALUE AND GRADIENT VECTOR FOR INITIAL ARGUMENT 
*/
    /* Parameter adjustments */
    --h;
    --g;
    --x;

    /* Function Body */
    sumsq(n, &x[1], f, &g[1], &ieri);
/* JP */
    if (ieri == 3) {
/* JP */
	*ier = 3;
/* JP */
	return 0;
/* JP */
    }

/*        RESET ITERATION COUNTER AND GENERATE IDENTITY MATRIX */
/* JP */
    *ier = 0;
    kount = 0;
    n2 = *n + *n;
    n3 = n2 + *n;
    n31 = n3 + 1;
L1:
    k = n31;
    i__1 = *n;
    for (j = 1; j <= i__1; ++j) {
	h[k] = 1.;
	nj = *n - j;
	if (nj <= 0) {
	    goto L5;
	} else {
	    goto L2;
	}
L2:
	i__2 = nj;
	for (l = 1; l <= i__2; ++l) {
	    kl = k + l;
/* L3: */
	    h[kl] = 0.;
	}
/* L4: */
	k = kl + 1;
    }

/*        START ITERATION LOOP */
L5:
    ++kount;

/*        SAVE FUNCTION VALUE, ARGUMENT VECTOR AND GRADIENT VECTOR */
    oldf = *f;
    i__1 = *n;
    for (j = 1; j <= i__1; ++j) {
	k = *n + j;
	h[k] = g[j];
	k += *n;
	h[k] = x[j];

/*        DETERMINE DIRECTION VECTOR H */
	k = j + n3;
	t = 0.;
	i__2 = *n;
	for (l = 1; l <= i__2; ++l) {
	    t -= g[l] * h[k];
	    if (l - j >= 0) {
		goto L7;
	    } else {
		goto L6;
	    }
L6:
	    k = k + *n - l;
	    goto L8;
L7:
	    ++k;
L8:
	    ;
	}
/* L9: */
	h[j] = t;
    }

/*        CHECK WHETHER FUNCTION WILL DECREASE STEPPING ALONG H. */
    dy = 0.;
    hnrm = 0.;
    gnrm = 0.;

/*        CALCULATE DIRECTIONAL DERIVATIVE AND TESTVALUES FOR DIRECTION */
/*        VECTOR H AND GRADIENT VECTOR G. */
    i__1 = *n;
    for (j = 1; j <= i__1; ++j) {
	hnrm += (d__1 = h[j], abs(d__1));
	gnrm += (d__1 = g[j], abs(d__1));
/* L10: */
	dy += h[j] * g[j];
    }

/*        REPEAT SEARCH IN DIRECTION OF STEEPEST DESCENT IF DIRECTIONAL */
/*        DERIVATIVE APPEARS TO BE POSITIVE OR ZERO. */
    if (dy >= 0.) {
	goto L51;
    } else {
	goto L11;
    }

/*        REPEAT SEARCH IN DIRECTION OF STEEPEST DESCENT IF DIRECTION */
/*        VECTOR H IS SMALL COMPARED TO GRADIENT VECTOR G. */
L11:
    if (hnrm / gnrm - *eps <= 0.) {
	goto L51;
    } else {
	goto L12;
    }

/*        SEARCH MINIMUM ALONG DIRECTION H */

/*        SEARCH ALONG H FOR POSITIVE DIRECTIONAL DERIVATIVE */
L12:
    fy = *f;
    alfa = (*est - *f) * 2. / dy;
    ambda = 1.;

/*        USE ESTIMATE FOR STEPSIZE ONLY IF IT IS POSITIVE AND LESS THAN 
*/
/*        1. OTHERWISE TAKE 1. AS STEPSIZE */
    if (alfa <= 0.) {
	goto L15;
    } else {
	goto L13;
    }
L13:
    if (alfa - ambda >= 0.) {
	goto L15;
    } else {
	goto L14;
    }
L14:
    ambda = alfa;
L15:
    alfa = 0.;

/*        SAVE FUNCTION AND DERIVATIVE VALUES FOR OLD ARGUMENT */
L16:
    fx = fy;
    dx = dy;

/*        STEP ARGUMENT ALONG H */
    i__1 = *n;
    for (i = 1; i <= i__1; ++i) {
/* L17: */
	x[i] += ambda * h[i];
    }

/*        COMPUTE FUNCTION VALUE AND GRADIENT FOR NEW ARGUMENT */
    sumsq(n, &x[1], f, &g[1], &ieri);
/* JP */
    if (ieri == 3) {
/* JP */
	*ier = 3;
/* JP */
	return 0;
/* JP */
    }
/* JP */
    fy = *f;

/*        COMPUTE DIRECTIONAL DERIVATIVE DY FOR NEW ARGUMENT.  TERMINATE 
*/
/*        SEARCH, IF DY IS POSITIVE. IF DY IS ZERO THE MINIMUM IS FOUND */
    dy = 0.;
    i__1 = *n;
    for (i = 1; i <= i__1; ++i) {
/* L18: */
	dy += g[i] * h[i];
    }
    if (dy < 0.) {
	goto L19;
    } else if (dy == 0) {
	goto L36;
    } else {
	goto L22;
    }

/*        TERMINATE SEARCH ALSO IF THE FUNCTION VALUE INDICATES THAT */
/*        A MINIMUM HAS BEEN PASSED */
L19:
    if (fy - fx >= 0.) {
	goto L22;
    } else {
	goto L20;
    }

/*        REPEAT SEARCH AND DOUBLE STEPSIZE FOR FURTHER SEARCHES */
L20:
    ambda += alfa;
    alfa = ambda;
/*        END OF SEARCH LOOP */

/*        TERMINATE IF THE CHANGE IN ARGUMENT GETS VERY LARGE */
    if (hnrm * ambda - 1e10 <= 0.) {
	goto L16;
    } else {
	goto L21;
    }

/*        LINEAR SEARCH TECHNIQUE INDICATES THAT NO MINIMUM EXISTS */
L21:
    *ier = 2;
    return 0;

/*        INTERPOLATE CUBICALLY IN THE INTERVAL DEFINED BY THE SEARCH */
/*        ABOVE AND COMPUTE THE ARGUMENT X FOR WHICH THE INTERPOLATION */
/*        POLYNOMIAL IS MINIMIZED */
L22:
    t = 0.;
L23:
    if (ambda != 0.) {
	goto L24;
    } else {
	goto L36;
    }
L24:
    z = (fx - fy) * 3. / ambda + dx + dy;
/* Computing MAX */
    d__1 = abs(z), d__2 = abs(dx), d__1 = max(d__1,d__2), d__2 = abs(dy);
    alfa = max(d__1,d__2);
    dalfa = z / alfa;
    dalfa = dalfa * dalfa - dx / alfa * dy / alfa;
    if (dalfa >= 0.) {
	goto L25;
    } else {
	goto L51;
    }
L25:
    w = alfa * sqrt(dalfa);
    alfa = dy - dx + w + w;
    if (alfa != 0.) {
	goto L250;
    } else {
	goto L251;
    }
L250:
    alfa = (dy - z + w) / alfa;
    goto L252;
L251:
    alfa = (z + dy - w) / (z + dx + z + dy);
L252:
    alfa *= ambda;
    i__1 = *n;
    for (i = 1; i <= i__1; ++i) {
/* L26: */
	x[i] += (t - alfa) * h[i];
    }

/*        TERMINATE, IF THE VALUE OF THE ACTUAL FUNCTION AT X IS LESS */
/*        THAN THE FUNCTION VALUES AT THE INTERVAL ENDS. OTHERWISE REDUCE 
*/
/*        THE INTERVAL BY CHOOSING ONE END-POINT EQUAL TO X AND REPEAT */
/*        THE INTERPOLATION.  WHICH END-POINT IS CHOOSEN DEPENDS ON THE */
/*        VALUE OF THE FUNCTION AND ITS GRADIENT AT X */

    sumsq(n, &x[1], f, &g[1], &ieri);
    if (ieri == 3) {
/* JP */
	*ier = 3;
/* JP */
	return 0;
/* JP */
    }
/* JP */
    if (*f - fx <= 0.) {
	goto L27;
    } else {
	goto L28;
    }
L27:
    if (*f - fy <= 0.) {
	goto L36;
    } else {
	goto L28;
    }
L28:
    dalfa = 0.;
    i__1 = *n;
    for (i = 1; i <= i__1; ++i) {
/* L29: */
	dalfa += g[i] * h[i];
    }
    if (dalfa >= 0.) {
	goto L33;
    } else {
	goto L30;
    }
L30:
    if ((d__1 = *f - fx) < 0.) {
	goto L32;
    } else if (d__1 == 0) {
	goto L31;
    } else {
	goto L33;
    }
L31:
    if (dx - dalfa != 0.) {
	goto L32;
    } else {
	goto L36;
    }
L32:
    fx = *f;
    dx = dalfa;
    t = alfa;
    ambda = alfa;
    goto L23;
L33:
    if (fy - *f != 0.) {
	goto L35;
    } else {
	goto L34;
    }
L34:
    if (dy - dalfa != 0.) {
	goto L35;
    } else {
	goto L36;
    }
L35:
    fy = *f;
    dy = dalfa;
    ambda -= alfa;
    goto L22;

/*        TERMINATE, IF FUNCTION HAS NOT DECREASED DURING LAST ITERATION 
*/
L36:
    if (oldf - *f + *eps >= 0.) {
	goto L38;
    } else {
	goto L51;
    }

/*        COMPUTE DIFFERENCE VECTORS OF ARGUMENT AND GRADIENT FROM */
/*        TWO CONSECUTIVE ITERATIONS */
L38:
    i__1 = *n;
    for (j = 1; j <= i__1; ++j) {
	k = *n + j;
	h[k] = g[j] - h[k];
	k = *n + k;
/* L37: */
	h[k] = x[j] - h[k];
    }

/*        TEST LENGTH OF ARGUMENT DIFFERENCE VECTOR AND DIRECTION VECTOR 
*/
/*        IF AT LEAST N ITERATIONS HAVE BEEN EXECUTED. TERMINATE, IF */
/*        BOTH ARE LESS THAN  EPS */
    *ier = 0;
    if (kount - *n >= 0) {
	goto L39;
    } else {
	goto L42;
    }
L39:
    t = 0.;
    z = 0.;
    i__1 = *n;
    for (j = 1; j <= i__1; ++j) {
	k = *n + j;
	w = h[k];
	k += *n;
	t += (d__1 = h[k], abs(d__1));
/* L40: */
	z += w * h[k];
    }
    if (hnrm - *eps <= 0.) {
	goto L41;
    } else {
	goto L42;
    }
L41:
    if (t - *eps <= 0.) {
	goto L56;
    } else {
	goto L42;
    }

/*        TERMINATE, IF NUMBER OF ITERATIONS WOULD EXCEED  LIMIT */
L42:
    if (kount - *limit >= 0) {
	goto L50;
    } else {
	goto L43;
    }

/*        PREPARE UPDATING OF MATRIX H */
L43:
    alfa = 0.;
    i__1 = *n;
    for (j = 1; j <= i__1; ++j) {
	k = j + n3;
	w = 0.;
	i__2 = *n;
	for (l = 1; l <= i__2; ++l) {
	    kl = *n + l;
	    w += h[kl] * h[k];
	    if (l - j >= 0) {
		goto L45;
	    } else {
		goto L44;
	    }
L44:
	    k = k + *n - l;
	    goto L46;
L45:
	    ++k;
L46:
	    ;
	}
	k = *n + j;
	alfa += w * h[k];
/* L47: */
	h[j] = w;
    }

/*        REPEAT SEARCH IN DIRECTION OF STEEPEST DESCENT IF RESULTS */
/*        ARE NOT SATISFACTORY */
    if (z * alfa != 0.) {
	goto L48;
    } else {
	goto L1;
    }

/*        UPDATE MATRIX H */
L48:
    k = n31;
    i__1 = *n;
    for (l = 1; l <= i__1; ++l) {
	kl = n2 + l;
	i__2 = *n;
	for (j = l; j <= i__2; ++j) {
	    nj = n2 + j;
	    h[k] = h[k] + h[kl] * h[nj] / z - h[l] * h[j] / alfa;
/* L49: */
	    ++k;
	}
    }
    goto L5;
/*        END OF ITERATION LOOP */

/*        NO CONVERGENCE AFTER  LIMIT  ITERATIONS */
L50:
    *ier = 1;
    return 0;

/*        RESTORE OLD VALUES OF FUNCTION AND ARGUMENTS */
L51:
    i__2 = *n;
    for (j = 1; j <= i__2; ++j) {
	k = n2 + j;
/* L52: */
	x[j] = h[k];
    }
    sumsq(n, &x[1], f, &g[1], &ieri);
    if (ieri == 3) {
/* JP */
	*ier = 3;
/* JP */
	return 0;
/* JP */
    }

/*        REPEAT SEARCH IN DIRECTION OF STEEPEST DESCENT IF DERIVATIVE */
/*        FAILS TO BE SUFFICIENTLY SMALL */
/* JP */
    if (gnrm - *eps <= 0.) {
	goto L55;
    } else {
	goto L53;
    }

/*        TEST FOR REPEATED FAILURE OF ITERATION */
L53:
    if (*ier >= 0) {
	goto L54;
    } else {
	goto L56;
    }
L54:
    *ier = -1;
    goto L1;
L55:
    *ier = 0;
L56:
/* JP */
    return 0;
/* JP */
} /* dfmfpc */

