/*   BEM = Batch Equilibrium Model (calculates chemical equilibira)
     Copyright (C) 1996  J. Prenzel (jprenze@gwdg.de)
     
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
     as published by the Free Software Foundation; either version 2
     of the License, or (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         */

/*----------------------------------------- lin/spline/lin ------------------*/
/*  linear-spline-linear 
kubischer Spline fuer ein Intervall mit linearer Fortsetzung links und rechts

(x1,y1), (x2,y2)=Grenzpunkte, d1 und d2 die 1. Ableitungen dort  */


static double x1,x2,y1,y2,d1,d2,u1,u2,a0,a1,a2,a3;
 
int setlspl(double par[]){

  x1=par[0];
  y1=par[1];
  d1=par[2];
  x2=par[3];
  y2=par[4];
  d2=par[5];
  
  u1=y2 - y1 - 0.5*(d2-d1)*(x1+x2) 
    - (x2-x1)*(d2-x2*(d2-d1)/(x2-x1));
  u2=x2*x2*x2 - x1*x1*x1 - 1.5*(x2*x2-x1*x1)*(x1+x2)
    + (x2-x1)*(3*x1*x2);
  a3=u1/u2;
  a2=0.5*(d2-d1)/(x2-x1) - 1.5*a3*(x2+x1);
  a1=d2-3*a3*x2*x2-2*a2*x2;
  a0=y2-a3*x2*x2*x2-a2*x2*x2-a1*x2;
  
  return 0;
}

int getlspl(double x, double *f, double *df){
  
  if(x < x1){
    *df=d1;
    *f=y1+(x-x1)*d1;
    return 0;
  }
  if(x > x2){
    *df=d2;
    *f=y2+(x-x2)*d2;
    return 0;
  }
  *f=a3*x*x*x + a2*x*x + a1*x + a0;
  *df=3*a3*x*x + 2*a2*x + a1;
  return 0;
}
